
%   This script file contains additional experiments related to
%   the paper
%     "A Texton for Fast and Flexible Gaussian Texture Synthesis." 
%       (Bruno Galerne, Arthur Leclaire, Lionel Moisan), 
%        preprint MAP5 n°2014-08.


%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ALGORITHM 2 - MULTIPLE INITIALIZATIONS %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% In this experiment, we investigate the possibility of running
%   algorithm 2 for SOT computation with multiple RPN initializations.

clear all
close all

% % Execute the following lines to run the Monte-Carlo simulation
%
% % Initialize random seed
% RandStream.setDefaultStream(RandStream('mt19937ar','seed',1));
% % Original image
% u = im2double(rgb2gray(imread('textures/IMG_7396.png')));
% u = u(1:256,1:384);
% [M,N] = size(u);
% figure
% imshow(u)
% title('Original texture')
% % Parameters
% niter = 10000;
% options.err = 0;
% options.nmax = niter;
% % Choice of a circular support
% renv = 15;
% M2 = floor(M/2)+1; N2 = floor(N/2)+1;
% sup = disc(M,N,M2,N2,renv,1),[1-M2 1-N2];
% sup(sup>0) = Inf;
% xind = max(sup,[],2)>0;
% yind = max(sup)>0;
% % Computation of several SOTs with RPN initializations
% nb = 100;
% dot = zeros(nb,niter);
% err = zeros(nb,niter);
% for k=1:nb
%     disp(num2str(k))
%     [~,evodot,evoerr] = so_texton(u,sup,options);
%     dot(k,:) = evodot;
%     err(k,:) = evoerr;
% end

% Or load the following file containing the data
load vary_tex6_ostexton_iter10000.mat

P = 10000;
n = 1;
r = 1;
clear xx q resdot
while (n<=niter)&&(P>=n)
    if (mod(P,n)==0)&&(P/n<=nb)
        xx(r) = n;
        q(r) = P/n;
        s = 0;
        Y = sort(dot(:,n));
%         % bootstrap without replacement
%         for t=1:nb-q(r)+1
%             s = s + nchoosek(nb-t,q(r)-1)*Y(t);
%         end
%         resdot(r) = s/nchoosek(nb,q(r));
        % bootstrap with replacement
        for t=1:nb
            s = s + ((nb-t+1)^(q(r)) - (nb-t)^(q(r)))*Y(t);
        end
        resdot(r) = s/nb^(q(r));
        r = r+1;
    end
    n = n+1;
end
nmax = n-1; 

ind = 1:length(xx);
gcf = figure;
hold on
plot(xx(ind),resdot(ind),'+','LineWidth',2)
xlabel('n')

%% 
%%%%%%%%%%%%%%%%%%%%%
% TEXTON RETRIEVAL  %
%%%%%%%%%%%%%%%%%%%%%

% In this experiment, we see that the "texton estimation" is
%  consistent for the disc texton example, given the true support.
% Also, it shows that the SOT computation can be done with a cropped
%  version of the covariance, which accelerates the algorithm.

clear all
close all

r = 5;
rb = 2*r; % size of the covariance
vt = disc(2*r+1,2*r+1,r+1,r+1,r,1);
% real covariance (for checking purpose)
vcov = disc(4*r+1,4*r+1,2*r+1,2*r+1,r,1);
vcov = real(ifft2(abs(fft2(vcov)).^2));
vcovs = fftshift(vcov);

% ADSN observation
factab = 10:10:100;
nobs = length(factab);

% Parameters of SO texton
options.err = 0;
options.nmax = 1000;
options.rho = 0;
options.step = 1;
options.intexton = 1;

% Stock the results
oterr = zeros(nobs,1);
loterr = zeros(nobs,1);
err = zeros(nobs,1);
lerr = zeros(nobs,1);
sizeobs = zeros(nobs,1);

for k=1:length(factab)

    nsamples = 10;
    for l=1:nsamples
        
        M = factab(k)*r; N = factab(k)*r;
        u = adsn(vt,zeros(M,N));
        % u = rpn(vt,zeros(M,N));
        mu = mean(u(:));
        % Canonical texton
        fcovper = 1/(M*N)*abs(fft2(u-mu)).^2;
        t0 = fftshift(real(ifft2(sqrt(fcovper))));
        % Cropped periodic estimator of covariance
        supcov = disc(M,N,M/2+1,N/2+1,rb,Inf);
        covper = real(ifft2(fcovper));
        covpers = fftshift(covper);
        covpers = covpers.*(supcov>0);
        covper = ifftshift(covpers);
        covpere = covpers(M/2+1-rb:M/2+1+rb,N/2+1-rb:N/2+1+rb);
        
        % Support desired for the textons
        sup = disc(M,N,M/2+1,N/2+1,r,Inf);
        
        % Central part of canonical texton
        t0sup = t0.*(sup>0);
        t0c = t0sup(M/2+1-r:M/2+1+r,N/2+1-r:N/2+1+r);
        
        % SOT computed with the canonical texton
        ost = so_texton(t0,sup,options);
        ostc = ost(M/2+1-r:M/2+1+r,N/2+1-r:N/2+1+r);
        if mean(ostc(:))<0
            ostc = -ostc;
        end
        
        % SOT computed with a cropped covariance
        lsup = disc(4*r+1,4*r+1,2*r+1,2*r+1,r,Inf);
        lcovsup = disc(4*r+1,4*r+1,2*r+1,2*r+1,2*r,Inf);
        flt2 = fft2(ifftshift(covpere));
        flt2(flt2<0) = 0;
        lt = real(ifft2(sqrt(flt2)));
        lost = so_texton(lt,lsup,options);
        lostc = lost(r+1:3*r+1,r+1:3*r+1);
        if mean(lostc(:))<0
            lostc = -lostc;
        end
        
        % disp(['model error on big domain : ' num2str(otdist(vt,ostc,M,N,1))]);
        % disp(['model error on little domain : ' num2str(otdist(vt,lostc,4*r+1,4*r+1,1))]);
        
        oterr(k) = oterr(k) + otdist(vt,ostc,M,N,1);
        loterr(k) = loterr(k) + otdist(vt,lostc,4*r+1,4*r+1,1);
        
        % For the l2 errors, one has to compare with fixed mean value.
        ostcp = ostc - mean(ostc(:)) + mean(vt(:));
        err(k) = err(k) + norm(vt(:)-ostc(:))/norm(vt(:));
        lostcp = lostc - mean(lostc(:)) + mean(vt(:));
        lerr(k) = lerr(k) + norm(vt(:)-lostcp(:))/norm(vt(:));
        
    end
    
    oterr(k) = oterr(k)/nsamples;
    err(k) = err(k)/nsamples;
    loterr(k) = loterr(k)/nsamples;
    lerr(k) = lerr(k)/nsamples;
    
sizeobs(k) = M;

end

%
figure;
plot(sizeobs,oterr,'+r',sizeobs,loterr,'ob','LineWidth',2)
xlabel('Size of the observation')
ylabel('RME')
legend({'SOT','SOT with cropped covariance'})

figure;
plot(sizeobs,err,'+r',sizeobs,lerr,'ob','LineWidth',2)
xlabel('Size of the observation')
ylabel('L2 error')
legend({'SOT','SOT with cropped covariance'})

% Comparison of the Fourier transforms
Ms = 512; Ns = Ms; % Number of points for DFT
%   BE CAREFUL ! Ms,Ns must not be less than the covariance size
%                   (i.e. 2*rb=4*r)
t0b = zeros(Ms,Ns);
vtb = zeros(Ms,Ns);
ostb = zeros(Ms,Ns);
lostb = zeros(Ms,Ns);
t0b(1:2*r+1,1:2*r+1) = t0c;
vtb(1:2*r+1,1:2*r+1) = vt;
ostb(1:2*r+1,1:2*r+1) = ostc;
lostb(1:2*r+1,1:2*r+1) = lostc;

fvt = fftshift(log(1+abs(fft2(vtb))));
fctc = fftshift(log(1+abs(fft2(t0b))));
fost = fftshift(log(1+abs(fft2(ostb))));
flost = fftshift(log(1+abs(fft2(lostb))));

figure
subplot(2,2,1)
imshow(fvt,[])
title('Real texton')
subplot(2,2,2)
imshow(fctc,[])
title(['Periodic cropped texton, RME = ' ...
    num2str(otdist(vt,t0c,Ms,Ns,1))])
subplot(2,2,3)
imshow(fost,[])
title(['SO texton, RME = ' ...
    num2str(otdist(vt,ostb,Ms,Ns,1))])
subplot(2,2,4)
imshow(flost,[])
title(['SO texton (with cropped covariance), RME = ' ...
    num2str(otdist(vt,lostb,Ms,Ns,1))])

%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% THE SPECIAL CASE OF THE DISC TEXTON  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This experiment shows that the SOT computed by Algorithm 2 may
% not be optimal for DSN synthesis. Indeed, it is not clear that the
% convergence of the DSN towards its Gaussian limit is visually slower 
% when using the disc texton than when using the SOT.

clear all
close all

r = 10;
M = 256; N = 256;
M2 = floor(M/2)+1; N2 = floor(N/2)+1;
sup0 = disc(M,N,M2,N2,r,1);
cardsup0 = sum(sum(sup0));
maxu = 1/2 + 0.15/sqrt(var(sup0(:))); % in order to set std(u)=0.15
u0 = 1/2 + (maxu-1/2)*sup0;
u0n = 1/sqrt(M*N)*(u0 - mean(u0(:)));
u = rpn(u0n,1/2*ones(M,N));

mu = mean(u(:));
stdu = sqrt(var(u(:)));

figure
imshow(u)
title('Original texture')

% Canonical texton (with mean 0)
t = 1/sqrt(M*N)*fftshift(real(ifft2(abs(fft2(u-mu)))));

% Choice of a circular support
renv = 15; 
sup = disc(M,N,M2,N2,renv,Inf);
cardsup = sum(sum(sup>0));
% SO texton
options.err = 0;
options.nmax = 1000;
options.visu = 1;
sot = so_texton(u,sup,options);
if mean(sot(:))<0
    sot = -sot;
end
% Extraction of the central part
xind = max(sup,[],2)>0;
yind = max(sup)>0;
tsup = t.*(sup>0);
tc = tsup(xind,yind);
sotc = sot(xind,yind);
[m n] = size(tc);

% Synthesis
Ms = 256;
Ns = 256;

% DSN
nbop = 15; % Number of impacts per pixel
lambda = nbop/cardsup;
disp(['Intensity of DSNs with spots supported by sup : ' num2str(lambda)])

disct = double(disc(2*r+1,2*r+1,r+1,r+1,r,1));
disct = 1/sqrt(M*N)*(maxu-1/2)*disct;
lambdadisct = nbop/cardsup0;
disp(['Intensity of DSN with original disc : ' num2str(lambdadisct)])
% in order to allow same size visualization of textons :
disctb = 1/sqrt(M*N)*(u0(xind,yind)-1/2);

% DSNs
snt = dsn(tc,mu*ones(Ms,Ns),lambda);
snsot = dsn(sotc,mu*ones(Ms,Ns),lambda);
sndisct = dsn(disct,mu*ones(Ms,Ns),lambdadisct);
% ADSNs
adsnt = adsn(tc,mu*ones(Ms,Ns));
adsnsot = adsn(sotc,mu*ones(Ms,Ns));
adsndisct = adsn(disct,mu*ones(Ms,Ns));

% Texton normalization for visualization
sotvisu = 1/2+sqrt(m*n)*sot;

% Visualization
figure
subplot(2,4,1)
imshow(u)
title('Original texture')
subplot(2,4,2)
imshow(u0)
title('True texton')
subplot(2,4,3)
imshow(adsndisct)
title('ADSN with the original disc')
subplot(2,4,4)
imshow(sndisct)
title('DSN with the original disc')
subplot(2,4,6)
imshow(sotvisu)
title('SO texton')
subplot(2,4,7)
imshow(adsnsot)
title('ADSN with the SO texton')
subplot(2,4,8)
imshow(snsot)
title('DSN with the SO texton')
maximize
