function tcc = cov_correction( t,u )
%COV_CORRECTION Covariance correction for a spot noise synthesis texton.
%
%   tcc = cov_correction( t,u ) a transformation in the color space, so that
%   the covariance of the distribution of one pixel of ADSN(t)
%   equals the empirical covariance of u.
%
%   Author : Arthur Leclaire
%   v. 1.0 (03/2014) : first public version

    [M,N,C] = size(u);
    [m,n,c] = size(t);
    
    if c~=C
        error('t and u must have the same number of channels')
    end
    
    covu = zeros(c);
    mu = mean(mean(u),2);
    act = zeros(c);
    for i=1:c
        for j=1:c
            covu(i,j) = 1/(M*N)*...
                sum(sum( (u(:,:,i)-mu(i)).*(u(:,:,j)-mu(j))));
            act(i,j) = sum(sum(t(:,:,i).*t(:,:,j)));
        end
    end
    
    [vu,du] = eig(covu);
    [vt,dt] = eig(act);
        
    isqdt = zeros(c);
    isqdt(dt>0) = dt(dt>0).^(-1/2);
    
    P = vu*du.^(1/2)*isqdt*vt';
    
    % Perform the product
    tcc = reshape(t,[m*n c])';
    tcc = P*tcc;
    tcc = reshape(tcc',[m n c]);
  
end

